/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Map.hpp"

Map::Map(unsigned int x, unsigned int y, unsigned char h){
    size.x = x;
    size.y = y;
    content = new unsigned short*[x];
    for (unsigned int i=0; i<x; i++){
        content[i] = new unsigned short[y];
    }
    for (unsigned int i=0; i < x; i++) {
        for (unsigned int j=0; j < y; j++) {
            content[i][j] = h;
        }
    }
    waterlevel = h;
}

Map::~Map(){
    for (int i=0; i < size.x; i++){
        delete content[i];
    }
    delete content;
}

Map::Map(const Map& original){
    size = original.size;
    if(size.x <0 or size.y <0){
        throw Hexception("Invalid map size.");
    }
    content = new unsigned short*[size.x];
    for (int i=0; i < size.x; i++){
        content[i] = new unsigned short[size.y];
    }
    waterlevel = original.waterlevel;
    for (int x=0; x < size.x; x++){
    	for (int y=0; y < size.x; y++){
            content[x][y] = original.content[x][y];
    	}
    }
}

void Map::setHeight(unsigned int x, unsigned int y, unsigned char h){
	setHeight(Coordinates(x,y), h);
}

void Map::setHeight(Coordinates pos, unsigned char h){
    if (!(pos < Coordinates(0,0)) and (pos < size)){
        content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFF00 );
        content[pos.x][pos.y] += h;
    }
}

unsigned char Map::getWaterLevel(){
    return waterlevel;
}

TERRAINTYPE Map::getTerrain(Coordinates pos){
    if ((pos > Coordinates(-1,-1)) and (pos < size)){
        if(waterlevel > content[pos.x][pos.y]%0x100){
    	    return WATER;
        } else {
    	    return GRASS;
        }
    }
    return WATER;
}

void Map::updateSafetyStatus(){
    Coordinates p;
    for(p.x=0; p.x<size.x; (p.x)++){
        for(p.y=0; p.y<size.y; (p.y)++){
            if (getVisStatus(p) < CLEAR){
                setUnsafe(p + Coordinates(0,-1));
                setUnsafe(p + Coordinates(0,+1));
                setUnsafe(p + Coordinates(-1,0));
                setUnsafe(p + Coordinates(+1,0));
                if (p.x %2 == 0){
                    setUnsafe(p + Coordinates(-1,-1));
                    setUnsafe(p + Coordinates(+1,-1));
                } else {
                    setUnsafe(p + Coordinates(-1,+1));
                    setUnsafe(p + Coordinates(+1,+1));
                }
            }
        }
    }
}

void Map::setUnsafe(Coordinates pos){
    if (getVisStatus(pos) == CLEAR){
        setVisStatus(pos, EXPLORED);

        setUnsafe(pos + Coordinates(0,-1));
        setUnsafe(pos + Coordinates(0,+1));
        setUnsafe(pos + Coordinates(-1,0));
        setUnsafe(pos + Coordinates(+1,0));
        if (pos.x %2 == 0){
            setUnsafe(pos + Coordinates(-1,-1));
            setUnsafe(pos + Coordinates(+1,-1));
        } else {
            setUnsafe(pos + Coordinates(-1,+1));
            setUnsafe(pos + Coordinates(+1,+1));
        }
    }
}

void Map::updateVisDex(){
    visdex = 0;
    for (int i = 0; i < size.x; i++) {
        for (int j = 0; j < size.y; j++) {
            visdex += getVisStatus(i,j);
        }
    }
}

void Map::setVisStatus(Coordinates pos, VISIBILITY stat){
    if (!(pos < Coordinates(0,0)) and pos < size){
        content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFCFF);
        content[pos.x][pos.y] += stat * 0x100;
    }
}

void Map::setVisStatus(int x, int y, VISIBILITY stat){
    setVisStatus(Coordinates(x,y), stat);
}

void Map::upgradeVisStatus(Coordinates pos, VISIBILITY stat){
    if (stat > getVisStatus(pos)) {
        if (!(pos << Coordinates(0,0)) and pos < size){
            content[pos.x][pos.y] = (content[pos.x][pos.y] & 0xFCFF);
            content[pos.x][pos.y] += stat * 0x100;
        }
    }
}

void Map::upgradeVisStatus(int x, int y, VISIBILITY stat ){
    upgradeVisStatus(Coordinates(x,y), stat);
}

VISIBILITY Map::getVisStatus(Coordinates pos){
    if (!(pos << Coordinates(0,0)) and pos < size){
        return ((content[pos.x][pos.y] & 0x0300)/0x100);
    } else {
        return UNKNOWN;
    }
}


VISIBILITY Map::getVisStatus(int x, int y){
    return getVisStatus(Coordinates(x,y));
}

void Map::sync(Map* orig){
    for (int i=0; i < size.x; i++) {
        for (int j=0; j < size.y; j++) {
            content[i][j] = (orig->content[i][j] & 0x00FF) + (content[i][j] & 0xFF00);
        }
    }
}

std::string Map::getAscii(){
    std::stringstream out;
    for (int i=0; i < size.x; i++) {
        for (int j=0; j < size.y; j++) {
            if (i%2 == 1) {
                out << ' ';
            }
            out << (int) getHeight(i,j);
            if (i%2 == 0) {
                out << ' ';
            }
        }
        out << '\n';
    }
    return out.str();
}

