/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* The class 'Map' contains the maps the
 * the game is played on. The information 
 * that need to be saved are the size of 
 * the map, a heightmap and the height of the
 * water in the level. */

#ifndef _MAP_H
#define	_MAP_H


typedef unsigned int TERRAINTYPE;
enum Terrains{
    WATER =0, GRASS =1, BLACK
};

typedef unsigned char VISIBILITY;
enum visstatus{
    UNKNOWN = 0, EXPLORED = 1, CLEAR = 2, VISIBLE = 3
};

#include  <sstream>

#include "Coordinates.hpp"
#include "../misc/Exceptions.hpp"

class Map{
public:
    /** deletes the Map and frees all memory that
     *  was requested by the object
     */
    ~Map();
	
    /** creates a map of x*y fields;
     *  h specifies the water level;
     *  all fields will contain land
     *  at waterlevel
     */
    Map(unsigned int x, unsigned  int y, unsigned char h);

    /// creates a new Map as a copy of another one
    Map(const Map& original);

    /// \returns the size of the Map
    Coordinates getSize(){
        return size;
    }
    
    /// \returns the height of the specified Coordinates
    unsigned char getHeight(unsigned int x, unsigned int y){
        return content[x][y]%0x100;
    }
    unsigned char getHeight(Coordinates f){
        return getHeight(f.x,f.y);
    }
    /// lets you set the height of one field.
    void setHeight(unsigned int x, unsigned int y, unsigned char h);
    void setHeight(Coordinates, unsigned char h);

    /// \returns the waterlevel
    unsigned char getWaterLevel();

    /// \returns TERRAINTYPE (based on hightmap and waterlevel)
    TERRAINTYPE getTerrain(Coordinates);

    void updateVisDex();
    unsigned int getVisDex(){
        return visdex;
    }

    /** does a consistency check weather fields marked as
     *  CLEAR (save) are lying next do unsave fields and
     *  sets their status to EXPLORED
     */
    void updateSafetyStatus();

    void setVisStatus(Coordinates pos, VISIBILITY stat);
    inline void setVisStatus(int x, int y, VISIBILITY stat);

    void upgradeVisStatus(Coordinates pos, VISIBILITY stat);
    inline void upgradeVisStatus(int x, int y, VISIBILITY stat );

    VISIBILITY getVisStatus(Coordinates pos);
    inline VISIBILITY getVisStatus(int x, int y);

    void sync(Map* orig);

    /// gives you an ASCII version of the hightmap (transposed)
    std::string getAscii();

private:
    /// the size of the Map
    Coordinates size;

    /** this will contain the following:
     *  - heightmap (lowest 8 bit)
     *  - sight (middle 2 bit): contains VISABILITY
     *  - danger (highest 6 bit): 0 = everything is fine
     */
    unsigned short** content;


    /** This specifies the "mean sea level" of the Map.
     *  It may be used to be able to change it while
     *  playing or to customize standard maps in an easy
     *  way.
     */
    unsigned char waterlevel;

    /// visdex is a score for map controll
    unsigned int visdex;

    /** \short use to mark a field that cannot be
     *  considered safe
     *
     *  if this means a change in the safetylevel setUnafe
     *  will (recursively) set the neighboring fields
     *  unsafe as well
     */
    void setUnsafe(Coordinates pos);
};

#endif	/* _MAP_H */
