/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "MapLoader.hpp"

VirtualMapLoader::VirtualMapLoader() {
    map = NULL;
    player = NULL;
}

void VirtualMapLoader::loadMap(std::string levelname) {
    // delete current data before loading a new map
    delete map;
    delete player;

    PathName path = levelname;
    if (!fsh->exists(path)) {
        path = "maps/";
        path /= levelname;
    }

    try{
        path = fsh->getFullPath(path);
    }
    catch(Hexception& e){
        std::cerr << e.what();
        exit(1);
    }
    loadMapFromDisk(path);
}

void VirtualMapLoader::saveMap(std::string savegame) {
    prepareSavePath();

    PathName path = savepath;
    path /= savegame;

    saveMapToDisk(path);
}

Map* VirtualMapLoader::getMap(){
    return map;
}

void VirtualMapLoader::setMap(Map* omap) {
    map = omap;
}

UnitContainer* VirtualMapLoader::getUnits(){
    return player;
}

void VirtualMapLoader::setUnits(UnitContainer* ounits) {
    player = ounits;
}


/* HWmap_V01 format saves owner of a unit in
 * the field before the last digit of a decimal
 * number. */
int VirtualMapLoader::hw01_owner(int feld){
    return ((feld%100 - feld%10)/10);
}

/* HWmap_V01 format saves unit (and terrain!)
 * in the last digit of a decimal number. */
int VirtualMapLoader::hw01_unit(int feld){
    return (feld%10);
}

void VirtualMapLoader::parse_HWmap_V01(unsigned int tmpmap[32][32]) {
    unsigned int X = map->getSize().x;
    unsigned int Y = map->getSize().y;
    for (unsigned int x=0; x<X; x++){
        for (unsigned int y=0; y<Y; y++){
            /* HWmap_V01 only can handle Units of PLAYER1
             * and PLAYER2. PLAYER0 is reserved for Terrain. */
        if (hw01_owner(tmpmap[x][y]) != PLAYER0){
                switch (hw01_unit(tmpmap[x][y])){
                case SPIRIT:
                    player->saveUnit(new Spirit(      Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case MAGICIAN:
                    player->saveUnit(new Magician(    Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case SCOUT:
                    player->saveUnit(new Scout(       Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case TERRAFORMER:
                    player->saveUnit(new TerraFormer( Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                case WARRIOR:
                    player->saveUnit(new Warrior(     Coordinates(x,y),
                                                      hw01_owner(tmpmap[x][y])));
                    break;
                default:
                    break;
                }
            }
            /* HWmap_V01 saves water as 0, everything else
             * must be assumed as height 1 since units are
             * saved in the map. */
            if (tmpmap[x][y] == 0) {
                map->setHeight(x,y,0);
            } else {
                map->setHeight(x,y,1);
            }
        }
    }
}

void VirtualMapLoader::prepareSavePath() {
    savepath = fsh->getHomeDir();
    fsh->prepareDirectory(savepath);

    savepath /= "save/";
    fsh->prepareDirectory(savepath);
}
