/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __Al4BasicUserInterface_HPP__
#define __Al4BasicUserInterface_HPP__

class Al4BasicUserInterface;
typedef Al4BasicUserInterface BasicUserInterface;

#include <unistd.h>
#include <string>
#include <ostream>
#include <stdlib.h>
#include <fstream>

#include <allegro.h>

#include "../../misc/Exceptions.hpp"
#include "../../filesystem/PathName.hpp"
#include "../../filesystem/FSHandler.hpp"
#include "../../definitions/Coordinates.hpp"
#include "../UI.hpp"

/** \brief responsable for display and input operations
 *
 *  Al4BasicUserInterface is responsible for recognizing basic
 *  user inputs (defined in UI) and for managing Hexwars
 *  display or window. It provides the ground UI::buffer
 *  which is used as a double buffer.
 *
 */

class Al4BasicUserInterface : public UI{
public:
    Al4BasicUserInterface(FSHandler* ofsh);

    ~Al4BasicUserInterface();

    void setFullscreen(bool);
    bool getFullscreen();

    void setResolution(Coordinates, int);

    /** \brief Returns user input done by keys.
     *
     *  This will return one of the folowing:
     *  <UL><LI>LMB: left mouse button is clicked</LI>
     *  <LI>RMB: right mouse button is clicked</LI>
     *  <LI>UP: 'W' or 'KEY_UP' is pushed</LI>
     *  <LI>DOWN: 'S' or 'KEY_DOWN' is pushed</LI>
     *  <LI>SELECT: 'ENTER' or 'SPACE' is pushed</LI>
     *  <LI>NOINPUT (else, even if key is not pushed but held)</LI>
     *  <LI>QUIT: 'Q' or 'ESC' is pushed</LI></UL>
     *
     * WARNING: When you call getInput() from different places
     * the returned value might be different from what you
     * expect. Example: Function A calls getInput() and gets
     * LMB, then function B calls getInput() and gets LMBHOLD
     * without any LMB returned to function B before.
     *
     * \returns code for user input
     */
    INPUTTYPE getInput(INPUTTYPE currinput);

    /**
     * Sets the screen resolution, fullscreen, etc.
     * You need to call this after switching resolution
     * et al.
     */
    void setGraphicMode();

    void print();

    void saveUIConfig();
    
    void message(const char* msg){
       allegro_message(msg);
    }

private:
    /** fullscreen */
    bool SFull;

    /** color depth */
    int SDepth;

    /** loads config file for UI config data,
     *  it includes default resolution et al.
     */
    void loadUIConfig();

    void drawPointer();

    FSHandler* fsh;

    PathName uicfgpath;

    void prepareUICfgPath();
};

#endif /* __Al4BasicUserInterface_HPP__ */
