/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Al5BasicUserInterface.hpp"

Al5BasicUserInterface::Al5BasicUserInterface(FSHandler* ofsh){
    fsh = ofsh;
    pos = Coordinates(0,0);
    screen = NULL;
    timer = NULL;
    event_queue = NULL;

    prepareUICfgPath();

    try{
        loadUIConfig();
    }
    catch(Hexception& e){
        // set default values
        SFull = false;
        size.x = 640;
        size.y = 480;
        SDepth = 16;
        FPS = 60;
        if (!uicfgpath.exists()){
            saveUIConfig();
            std::cout << "Warning: No configuration file found. Created default at '"
                      << uicfgpath.string() << "'.\n";
        } else {
            std::cerr << e.what();
        }
    }

    // initalize allegro addons
    al_init_primitives_addon();
    al_init_font_addon();
    al_init_ttf_addon();
    al_init_image_addon();

    //FIXME: Do not use absolute paths!
    font = al_load_ttf_font("/usr/share/fonts/liberation/LiberationSerif-Regular.ttf",16,0 );

    if (FPS < 10) {
        std::cerr << "Config says Hexwars should run with less than 10 FPS. "
                     "This doeas not make sense, I will set FPS to 15.";
        FPS = 15;
    }

    timer = al_create_timer(1.0 / FPS);
    if(!timer) {
        std::cerr << "Failed to create timer.\n";
        exit(-1);
    }

    event_queue = al_create_event_queue();
    if(!event_queue) {
         std::cerr << "Failed to create event_queue!\n";
         exit(-1);
     }

    // set screen resolution
    setGraphicMode();
    // make keyboard usable
    al_install_keyboard();
    // make mouse usable
    al_install_mouse();

    init(size, pos, NULL, NULL);

    al_register_event_source(event_queue, al_get_timer_event_source(timer));
    al_register_event_source(event_queue, al_get_keyboard_event_source());
    al_register_event_source(event_queue, al_get_mouse_event_source());

    al_start_timer(timer);

    //FIXME not needed for pure Al5 version

    ALLEGRO_PATH* exedir = al_get_standard_path(ALLEGRO_EXENAME_PATH);
    fsh->setExeDir( PathName(al_path_cstr(exedir,ALLEGRO_NATIVE_PATH_SEP)));
    al_destroy_path(exedir);

    lastinput = NOINPUT;
}

Al5BasicUserInterface::~Al5BasicUserInterface(){
    al_destroy_font(UI::font);
    al_destroy_display(screen);
    al_shutdown_primitives_addon();
    al_shutdown_ttf_addon();
    al_shutdown_font_addon();
    al_shutdown_image_addon();
}

void Al5BasicUserInterface::setGraphicMode(){
    if (screen != NULL) {
        al_unregister_event_source(event_queue, al_get_display_event_source(screen));
        al_destroy_display(screen);
        screen = NULL;
    }
    screen = al_create_display(size.x, size.y);
    if (!screen) {
        std::cerr << "Failed to create display.\n";
        exit(-1);
    }
    al_register_event_source(event_queue, al_get_display_event_source(screen));
    return;
}

INPUTTYPE Al5BasicUserInterface::getInput(INPUTTYPE currinput){
    ALLEGRO_EVENT ev;

    al_wait_for_event(event_queue, &ev);
    if (ev.type == ALLEGRO_EVENT_TIMER && al_event_queue_is_empty(event_queue)) {
        return checkMouseHeld();
    }
    if (ev.type == ALLEGRO_EVENT_KEY_DOWN) {
        switch (ev.keyboard.keycode) {

        case ALLEGRO_KEY_ESCAPE:
        case ALLEGRO_KEY_Q:
            return QUIT;

        case ALLEGRO_KEY_W:
        case ALLEGRO_KEY_UP:
            return UP;

        case ALLEGRO_KEY_S:
        case ALLEGRO_KEY_DOWN:
            return DOWN;

        case ALLEGRO_KEY_D:
        case ALLEGRO_KEY_RIGHT:
            return RIGHT;

        case ALLEGRO_KEY_A:
        case ALLEGRO_KEY_LEFT:
            return LEFT;

        case ALLEGRO_KEY_ENTER:
        case ALLEGRO_KEY_SPACE:
            return SELECT;

        case ALLEGRO_KEY_PAD_1:
            return KP1;

        case ALLEGRO_KEY_PAD_2:
            return KP2;

        case ALLEGRO_KEY_PAD_3:
            return KP3;

        case ALLEGRO_KEY_PAD_4:
            return KP4;

        case ALLEGRO_KEY_PAD_5:
            return KP5;

        case ALLEGRO_KEY_PAD_6:
            return KP6;

        case ALLEGRO_KEY_PAD_7:
            return KP7;

        case ALLEGRO_KEY_PAD_8:
            return KP8;

        case ALLEGRO_KEY_PAD_9:
            return KP9;
        default:
            break;
        }
    }
    if (ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {
        return QUIT;
    }
    if (ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_DOWN) {
        ALLEGRO_MOUSE_STATE state;
        al_get_mouse_state(&state);
        if (state.buttons & 1) {
            if (!(lastinput == LMB)) {
                lastinput = LMB;
                return LMB;
            }
        }
        if (state.buttons & 2) {
            if (!(lastinput == RMB)) {
                lastinput = RMB;
                return RMB;
            }
        }
        if (state.buttons & 4) {
            if (!(lastinput == MMB)) {
                lastinput = MMB;
                return MMB;
            }
        }
    }
    INPUTTYPE mouseheld = checkMouseHeld();
    if (mouseheld != NOINPUT) {
        return mouseheld;
    }
    if (ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_UP) {
        ALLEGRO_MOUSE_STATE state;
        al_get_mouse_state(&state);
        if (!(state.buttons & 1) and
            !(state.buttons & 2) and
            !(state.buttons & 4)) {
            lastinput = NOINPUT;
        }
    }
    return TIME;
}

void Al5BasicUserInterface::loadUIConfig(){
    std::ifstream uisettings;
    uisettings.open(uicfgpath.c_str());
    if(!uisettings.good()){
        throw Hexception( std::string("Error loading config file: '")
                          + uicfgpath.string()
                          + std::string("'. Will use default values.\n")
                        );
        return;
    }
    char tmpchar;
    std::string descripiton;
    while(uisettings.good()){
        descripiton.clear();
        for(int i=0; i<11 ; i++){
            uisettings >> tmpchar;
            descripiton.push_back(tmpchar);
        }
        if(descripiton == "sfullscreen")
            uisettings >> SFull;
        if(descripiton == "screenwidth")
            uisettings >> size.x;
        if(descripiton == "screenhight")
            uisettings >> size.y;
        if(descripiton == "scolordepth")
            uisettings >> SDepth;
        if(descripiton == "framepersec")
            uisettings >> FPS;
    }
        if(size.x <= 0 || size.y <= 0){
        throw Hexception( "Error: Invalid configuration. "
                          "Will use default values.\n");
        return;
    }
}

void Al5BasicUserInterface::saveUIConfig(){
    std::ofstream uisettings;
    uisettings.open(uicfgpath.c_str());
    uisettings << "screenwidth " << size.x << std::endl;
    uisettings << "screenhight " << size.y << std::endl;
    uisettings << "sfullscreen " << SFull << std::endl;
    uisettings << "scolordepth " << SDepth << std::endl;
    uisettings << "framepersec " << FPS <<std::endl;
    uisettings.close();
}

void Al5BasicUserInterface::setFullscreen(bool full){
    SFull = full;
}

void Al5BasicUserInterface::setResolution(int width, int hight, int depth){
    setResolution(Coordinates(width, hight), depth);
}

void Al5BasicUserInterface::setResolution(Coordinates res, int depth){
    size = res;
    SDepth = depth;
}

bool Al5BasicUserInterface::getFullscreen(){
    return SFull;
}

void Al5BasicUserInterface::print(){
    al_set_target_bitmap(al_get_backbuffer(screen));
    al_clear_to_color(al_map_rgb(0,0,0));
    al_draw_bitmap(TLB, .0, .0, 0);
    al_flip_display();
}

INPUTTYPE Al5BasicUserInterface::checkMouseHeld() {
    if ( lastinput == LMB or lastinput == LMBHOLD or
         lastinput == RMB or lastinput == RMBHOLD or
         lastinput == MMB or lastinput == MMBHOLD) {
        ALLEGRO_MOUSE_STATE state;
        al_get_mouse_state(&state);
        if (state.buttons & 1) {
            lastinput = LMBHOLD;
            return LMBHOLD;
        }
        if (state.buttons & 2) {
            lastinput = RMBHOLD;
            return RMBHOLD;
        }
        if (state.buttons & 4) {
            lastinput = MMBHOLD;
            return MMBHOLD;
        }
    }
    return NOINPUT;
}

void Al5BasicUserInterface::prepareUICfgPath(){
    uicfgpath = fsh->getHomeDir();
    fsh->prepareDirectory(uicfgpath);

    uicfgpath /= "cfg";
    fsh->prepareDirectory(uicfgpath);

    uicfgpath /= "uisettings.cfg";
}
