/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __Al5UI__HPP__
#define __Al5UI__HPP__

#include <vector>
#include <allegro5/allegro.h>
#include <allegro5/allegro_font.h>

#include "../UI.hpp"
#include "../../definitions/Coordinates.hpp"
#include "../menu/MenuItem.hpp"
#include "../GraphicSet.hpp"


/** \brief UIs are responsable for representing interaction
 * entities (like menus) to the user. Al5UIs can be seen as an
 * abstraction layer for several user interface solutions
 * (like allegro, X11, etc.)
 */
class Al5UI{
public:
    Al5UI();

    /** \brief Constructor needs parent Al5UI, input will be set to NULL
     *
     * size defines the size of the
     * Al5UI on screen, the constructor will create
     * a BITMAP for buffering in the given
     * size
     *
     * pos defines the position of the
     * Al5UI on the parents input
     *
     * UI needs a pathnameet* to be able to print
     * graphics, link one with gs
     *
     * UI* par defines a parent Al5UI, that this
     * UI will be printed on
     *
     * WARNING: input is set to NULL by constructor
     * because it is only needed if there are child
     * UIs
     */
    Al5UI(Coordinates size, Coordinates pos, GraphicSet* gs, Al5UI* par);

    /** \brief destroys bitmaps
     */
    ~Al5UI();

    void setGraphicSet(GraphicSet* gs);

    /** \brief Just needed for campatibily reasons */
    void lockScreen(bool doit) {
        return;
    }

    /** Allegro-5-version of Hexwars does not support mouse yet!
     *
     * \returns Coordinates(-1,-1)
     */
    Coordinates getPointerPos();

    /** \brief Prints buffer and input on parents input.
     *
     * First of all it prints scrollbuffer, the
     * staticbuffer (the one with transparency)
     * and the pointer on the screenbuffer. Then
     * the screenbuffer is printed on screen.
     */
    virtual void print();

    void clearBlack(){
        al_set_target_bitmap(buffer);
        al_clear_to_color(al_map_rgb(0,0,0));
    }

    /** \brief Clears the screen.
     *
     * Clears the buffer to magic pink.
     */
    void clear() {
        al_set_target_bitmap(buffer);
        al_clear_to_color(al_map_rgba(0,0,0,0));
    }

    /** \brief clears top layer buffer
     *
     */
    void clearTLB() {
        al_set_target_bitmap(TLB);
        al_clear_to_color(al_map_rgba(0,0,0,0));
    }

    /** \brief This method analyzes if user input is importent
     * for current UI.
     *
     * It will return NOINPUT if nothing important happened.
     * This basic implementation just returns the return HOVER
     * if pointer is above UI.
     *
     * \returns code for user input: HOVER or NOINPUT
     */
    virtual INPUTTYPE getInput(INPUTTYPE currinput);

    /** \brief moves UI around on parent Al5UI
     *
     */
    void changePos(Coordinates dp);

    /** \brief Returns the size of the window.
     *
     * Returns the size of the window that is
     * currently displayed.
     *
     * \returns size of the UI
     */
    Coordinates getSize();

    /** \brief change size on screen
     *
     */
    void setSize(Coordinates s);

    /** \brief Children will draw their buffers onto this BITMAP.
     *
     * This is the top layer buffer. If there are child Al5UIs they
     * will use this buffer to draw on parent UIs. After printing
     * its own doublebuffer on screen (or onto the parent TLB) this
     * input will be printed. This means that only contents that
     * really need a refresh will be refreshed.
     */
    ALLEGRO_BITMAP* TLB;

    void printText(std::string text, Coordinates pos);

    static ALLEGRO_FONT* font;

protected:
    /** \brief function for initialization
     * it is a good idea to use this in your constructor of child classes
     */
    void init(Coordinates size, Coordinates pos, GraphicSet* gs, Al5UI* par);

    /** \brief saves last input
     */
    INPUTTYPE lastinput;

    /** \brief saves the size of the Al5UI on parents input
     */
    Coordinates size;

    /** \brief saves the position of the Al5UI on parents input
     */
    Coordinates pos;

    /** \brief internal buffer — only redraw contents if refresh is needed
     */
    ALLEGRO_BITMAP* buffer;

    /** \brief storage container for single elements to draw on screen
     */
    GraphicSet* graphics;

    /** \brief every UI has a parent UI (parent window)
     *
     * Every Al5UI is nested in anoter Al5UI. The basis of this is a special
     * implementation of Al5UI, the BasicUserInterface. It has no parent
     * but will use pure allegro routines.
     */
    Al5UI* parent;
};

#endif /* __Al5UI__HPP__ */
