/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ENGINE__H__
#define __ENGINE__H__

#include <vector>

#include "../../definitions/Coordinates.hpp"
#include "../../definitions/Map.hpp"
#include "../../units/UnitContainer.hpp"
#include "../../units/Magician.hpp"
#include "../IE.hpp"
#include "UI_Engine.hpp"

/**
 * The Engine is an IE (Interaction Entity)
 * responsible for displaying map, units etc.
 * The user's input and output are actually
 * done by the UI_Engine. This is a genreric
 * Engine for Client and Editor. Specific
 * functions have to be implemeted there.
 */

class Engine : public IE{
public:
    /** The engine needs a map and units to display. */
    Engine(Map*, UnitContainer*, UI_Engine* uie, PLAYER oplyr);

    ~Engine();

    /**
     * refreshes all the contents seen on the screen
     * except the pointer. This will NOT draw anything
     * on the screen.
     */
    void refresh();

    /** gives you the currently selected field */
    Coordinates getSelectedField();

    /** centers the map on the given coordinates */
    void center(Coordinates);

    /// calcs VISABILITY for player ply
    void setPlayerVis(PLAYER ply);

private:
    /**
     *  a pointer to the map to display
     */
    Map* map;

    /** temporary save fo a Unit to draw */
    Unit* drawthis;
	
    /**
     *  used to check wich units are selectable.
     *  field of view is checked against plyr
     *  as well, so a wrongly set value results in a
     *  black (or fogy) window
     */
    PLAYER plyr;

    /**
     * decides what engine does with given
     * user interaction. Implementation of
     * IE::specificInput(INPUTTYPE)
     */
    void specificInput(INPUTTYPE parinput);

    /**
     *  a pointer to a vector containing all (known) units
     */
    UnitContainer* units;

    void drawLine(std::vector<Coordinates> line, int col);

    /** Gives the gui the command to display a field.
     *
     * \param type of terrain (e.g. GRASS)
     * \param pos coordinates of the field (not position on display!)
     */
    void drawTerrain(TERRAINTYPE type, Coordinates pos);

    /** Gives the gui the command to display a unit.
     *
     *  \param unit pointer to he Unit to show
     */
    void drawUnit(Unit* unit );
	
    /** Gives the gui the command to display a unit.
     *
     *  \param type of the Unit (or 'FOG', e.g. WARRIOR)
     *  \param p of the unit (e.g. PLAYER1)
     *  \param pos coordinates where the unit is shown (in fields,
     *  not position on the screen)
     */
    void drawUnit(UNITTYPE type, PLAYER p, Coordinates pos);

    /** Gives the gui the command to display a selection. */
    void drawFokus(FOCUSTYPE type, Coordinates pos);

    /** Use this to calculate positions on screen from
     *  locations on map. */
    Coordinates displayPos(Coordinates mappos);

    Coordinates mapPos(Coordinates displaypos);

    /** This saves the current position (upper left corner)
     * on the map that is shown on the screen */
    Coordinates viewport;

    /** saves the highes value for viewport */
    Coordinates maxviewport;

    /** allow or lock horizontal scrolling */
    bool scroll_x;

    /** allow or lock vertical scrolling */
    bool scroll_y;

    /** This saves the size of the screen. */
    Coordinates screensize;

    /** This saves the size of one tile. */
    Coordinates tilesize;


    /** The size of the map (in tiles) */
    Coordinates mapsize;

    /** temporary save for positions */
    Coordinates p1;
    /** temporary save for positions */
    Coordinates p2;

    /** This saves the position of the pointer
     * (on the screen). */
    Coordinates pointerpos;

    /** This saves the position of the pointer
     * (on the map). */
    Coordinates selectedfield;
    Coordinates lastselectedfield;
    Coordinates movefield;

    /** Scrolls the displayed area based on pointerpos,
     * viewport and size of the map. Will return if
     * scrolling was required. */
    bool scroll();

    /** Selects something on the screen */
    bool select();

    /** cache for selected unit */
    Unit* selectedunit;

    /** second pointer to ui
     *  both pointers must point to the same UI_Engine
     *  to make the Engine work
     */
    UI_Engine* display;

    /** gives units orders to move based on pressed
     *  number on numpad
     */
    void giveOrder(INPUTTYPE kp);

    std::vector<Coordinates> way;
 };
 
 #endif /* __ENGINE_H__ */
 
