/* Hexwars - A Hexagonal Turn Based Strategy Game
 * Copyright (C) 2009-2011  Patrik Schönfeldt
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "Unit.hpp"

Unit::Unit(){
    type = NOUNIT;
    maxhp = 0;
    uuid = 0;
    speed = 0;
    sight = 0;
    map = NULL;
}

Unit::Unit(const Unit& orig){
    init(orig.pos, orig.owner);
    type = orig.type;
    uuid = orig.uuid;
    speed = orig.speed;
    sight = orig.sight;
    map = orig.map;
    hp = orig.hp;
    oldhp = orig.oldhp;
    maxhp = orig.maxhp;
}

void Unit::init(Coordinates opos, PLAYER player) {
    uuid = 0;
    oldhp = hp;
    pos = opos;
    owner = player;
    oldway.push_back(pos);
}

unsigned char Unit::getSight(){
    return sight;
}

unsigned char Unit::getSpeed() {
    return speed;
}

Coordinates Unit::getPos(){
    return pos;
}

void Unit::command(Coordinates target){
    if (isAlive()) {
        if (way.size() == 0){
            if(target.distanceTo(pos) == 1){
                way.push_back(target);
            }
        } else {
            if(target.distanceTo(way.back()) == 1){
                way.push_back(target);
            }
        }
    }
}

void Unit::clearCommands() {
    way.clear();
}

bool Unit::isAlive(){
    return (hp > 0);
}

void Unit::move(unsigned int time) {
    for (unsigned int i = 100; i <= speed*100; i+=100) {
        /* time runs from 1 to 100, the folowing
         * equation simulates constant movement
         * of all units (slow and fast ones)
         */
        if ((i / (speed + 1)) == time) {
            oldway.push_back(pos);
            if (way.size() > 0) {
                pos = way.front();
                way.erase(way.begin());
                /* FIXME: result depends on prior
                 * movement of terraformers
                 */
                if(map->getTerrain(pos) == WATER){
                    kill();
                }
            }
            return;
        }
    }
    return;
}

short Unit::getHp() {
    return hp;
}

short Unit::getOldHp() {
    return oldhp;
}

short Unit::getMaxHp() {
    return maxhp;
}

void Unit::newRoundPoke(){
    oldhp = hp;
}

void Unit::changeHp(short val) {
    if (isAlive()) {
        hp += val;
    }
    if (hp < 0){
        hp = 0;
    }
    if (hp > maxhp){
        hp = maxhp;
    }
}

void Unit::kill(){
    if (hp > 0){
        hp = 0;
    } else {
        hp--;
    }
}

unsigned int Unit::getUuid(){
    return uuid;
}

PLAYER Unit::getOwner(){
    return owner;
}

void Unit::setUuid(unsigned int id){
    if(uuid == 0){
        uuid = id;
    }
}

#ifdef DEBUG
void Unit::check() {
    if (map == NULL) {
        throw Hexception("Unit does not know map.");
    }
}
#endif

void Unit::setMap(Map* m){
    map = m;
}

std::vector<Coordinates> Unit::getWay(){
    return way;
}

std::vector<Coordinates> Unit::getOldWay(){
    return oldway;
}

void Unit::delOldWay(){
    oldway.clear();
    oldway.push_back(pos);
}

UNITTYPE Unit::getType(){
    return type;
}

void Unit::sync(Unit* orig){
    hp = orig->hp;
    pos = orig->pos;
    way = orig->way;
    oldway = orig->oldway;
    oldhp = orig->oldhp;
}

std::string Unit::getAsciiInfos(){
    std::stringstream output;
    output << uuid  << ";";
    output << pos.x << "," << pos.y << ";";
    output << type  << ";";
    output << hp    << ";";
    output << oldhp << ";";

    for(int i= (int) oldway.size()-1; i>=0; i--){
        output << oldway[i].x << "," << oldway[i].y << ";";
    }
    output << pos.x << "," << pos.y << ";";
    for(unsigned int i=0; i<way.size(); i++){
        output << way[i].x << "," << way[i].y << std::endl;
    }

    return output.str();
}
